use std::convert::{TryFrom, TryInto};
use serde::{Deserialize, Serialize};
use trust_dns_proto::error::ProtoError;

use super::trust_dns_types;
use super::name::SerdeName;
use super::class::DNSClass;
use super::rdata::RData;


#[derive(Deserialize, Serialize, Clone)]
pub struct Record {
    #[serde(rename = "Name")]
    pub name: SerdeName,
    // TODO: Make class optional, default to IN
    #[serde(rename = "Class")]
    pub dns_class: DNSClass,
    #[serde(rename = "TTL")]
    pub ttl: u32,
    #[serde(flatten)]
    pub rdata: RData,
}

impl From<trust_dns_types::Record> for Record {
    fn from(record: trust_dns_types::Record) -> Record {
        Record {
            name: SerdeName(record.name().clone()),
            dns_class: record.dns_class().into(),
            ttl: record.ttl(),
            rdata: record.into_data().into(),
        }
    }
}

impl TryFrom<Record> for trust_dns_types::Record {
    type Error = ProtoError;

    fn try_from(record: Record) -> Result<Self, Self::Error> {
        let mut trust_dns_record = trust_dns_types::Record::from_rdata(record.name.into_inner(), record.ttl, record.rdata.try_into()?);
        trust_dns_record.set_dns_class(record.dns_class.into());
        Ok(trust_dns_record)
    }
}